@@ -3,7 +3,7 @@
 
 ## ---- eval= F------------------------------------------------------------
 #  install.packages("devtools")
-#  devtools::install_github("agricolamz/lingtypology", dependencies = TRUE)
+#  devtools::install_github("agricolamz/lingtypology")
 
 ## ------------------------------------------------------------------------
 library(lingtypology)
@@ -49,6 +49,12 @@ is.glottolog(c("Tabasaran", "Tabassaran"), g = "m")
 ## ---- fig.width=6.2------------------------------------------------------
 map.feature(c("Adyghe", "Kabardian", "Polish", "Russian", "Bulgarian"))
 
+## ---- eval = FALSE-------------------------------------------------------
+#  m <- map.feature(c("Adyghe", "Korean"))
+#  # install.packages("htmlwidgets")
+#  library(htmlwidgets)
+#  saveWidget(m, file="/home/agricolamz/_DATA/OneDrive1/_Work/github/lingtypology/m.html")
+
 ## ------------------------------------------------------------------------
 df <- data.frame(language = c("Adyghe", "Kabardian", "Polish", "Russian", "Bulgarian"),
                  features = c("polysynthetic", "polysynthetic", "fusional", "fusional", "fusional"))
@@ -57,6 +63,10 @@ df
 ## ---- fig.width=6.2------------------------------------------------------
 map.feature(languages = df$language, features = df$features)
 
+## ---- fig.width=6.2------------------------------------------------------
+df$features <- factor(df$features, levels = c("polysynthetic", "fusional"))
+map.feature(languages = df$language, features = df$features)
+
 ## ---- fig.width=6.2------------------------------------------------------
 set.seed(42)
 map.feature(languages = df$language, features = df$features)
@@ -87,6 +97,32 @@ df$popup <- c ("sɐ s-ɐ-k'ʷɐ<br> 1sg 1sg.abs-dyn-go<br>'I go'",
 set.seed(42)
 map.feature(df$language, df$features, df$popup)
 
+## ---- fig.width=6.2------------------------------------------------------
+# Lets create a dataframe with links to video
+sign_df <- data.frame(languages = c("American Sign Language", "Russian Sign Language", "French Sign Language"),
+                 popup = c("https://media.spreadthesign.com/video/mp4/13/48600.mp4", "https://media.spreadthesign.com/video/mp4/12/17639.mp4", "https://media.spreadthesign.com/video/mp4/10/17638.mp4"))
+
+# Change popup to an HTML code
+sign_df$popup <- paste("<video width='200' height='150' controls> <source src='",
+                  as.character(sign_df$popup),
+                  "' type='video/mp4'></video>", sep = "")
+
+# create a map
+map.feature(languages = sign_df$languages, popup = sign_df$popup)
+
+## ---- fig.width=6.2------------------------------------------------------
+set.seed(42)
+map.feature(df$language, df$features,
+            label = df$language)
+
+## ---- fig.width=6.2------------------------------------------------------
+set.seed(42)
+map.feature(df$language, df$features,
+            label = df$language,
+            label.fsize = 20,
+            label.position = "left",
+            label.hide = TRUE)
+
 ## ---- fig.width=6.2------------------------------------------------------
 set.seed(42)
 map.feature(languages = circassian$language,
@@ -163,6 +199,12 @@ map.feature(circassian$language,
             longitude = circassian$longitude,
             title = "Circassian dialects", stroke.title = "Languages")
 
+## ---- fig.width=6.2------------------------------------------------------
+set.seed(42)
+map.feature(c("Adyghe", "Polish", "Kabardian", "Russian"),
+            scale.bar = TRUE,
+            scale.bar.position = "topright")
+
 ## ---- fig.width=6.2------------------------------------------------------
 df <- data.frame(lang = c("Adyghe", "Kabardian", "Polish", "Russian", "Bulgarian"),
    feature = c("polysynthetic", "polysynthetic", "fusion", "fusion", "fusion"),
@@ -197,6 +239,19 @@ map.feature(df$lang, df$feature, df$popup,
             tile = c("OpenStreetMap.BlackAndWhite", "Thunderforest.OpenCycleMap"),
             control = TRUE)
 
+## ---- fig.width=6.2------------------------------------------------------
+set.seed(42)
+map.feature(c("Adyghe", "Polish", "Kabardian", "Russian"),
+            minimap = TRUE)
+
+## ---- fig.width=6.2------------------------------------------------------
+set.seed(42)
+map.feature(c("Adyghe", "Polish", "Kabardian", "Russian"),
+            minimap = TRUE,
+            minimap.position = "topright",
+            minimap.height = 100,
+            minimap.width = 100)
+
 ## ---- fig.width=6.2------------------------------------------------------
 df <- data.frame(lang = c("Russian", "Russian"),
                  lat  = c(55.75, 59.95),
